@@ -4,23 +4,31 @@ import hashlib
 import os
 
 import ioc
-from django.apps import apps
-from django.db import transaction
 
 
-class AbstractBlobRepository:
+class AbstractBlobRepository(metaclass=abc.ABCMeta):
     """Provides an interface to persist blobs used as input for jobs (environment,
     arguments, secrets and files).
     """
     storage = ioc.class_property('octet.BlobStorage')
-    model_class = abc.abstractproperty()
+    DoesNotExist = abc.abstractproperty()
+    model = abc.abstractproperty()
 
-    @property
-    def model(self):
-        """Returns the model class used by this repository."""
-        return apps.get_model(self.model_class)
+    @classmethod
+    def class_factory(cls, impl_class):
+        """Create a new :class:`AbstractBlobRepository` subclass using the
+        provided implementation class `impl_class`.
+        """
+        return type('BlobRepository', (impl_class, cls), {})
+
+    @classmethod
+    def new(cls, impl_class, *args, **kwargs):
+        """Create a new instance with the specified implementation class."""
+        return cls.class_factory(impl_class)(*args, **kwargs)
+
+    def __init__(self):
+        self._in_transaction = False
 
-    @transaction.atomic
     @ioc.inject('factory', 'BlobFactory')
     def add(self, f, content_type, factory, encrypt=False):
         """Check if a :class:`~unimatrix.ext.octet.models.Blob` instance exists
@@ -31,32 +39,53 @@ class AbstractBlobRepository:
         checksum = self.calculate_checksum(f)
         created = False
         if not self.exists(checksum):
-            blob = factory.new(f, content_type, encrypt=encrypt)
+            blob = factory.new(self.model, f, content_type, encrypt=encrypt)
             created = True
         else:
-            blob = self.model.objects.get(checksum=checksum)
+            blob = self.lookup_dao(checksum)
         if created:
             labels = {}
             self.storage.push(f.name, blob.checksum)
             self.storage.label(labels)
             blob.setlabels(labels)
-            blob.save()
+            self.persist_dao(blob)
             blob.fd = f
             blob.fd.seek(0)
         return blob
 
+    async def async_lookup_dao(self, checksum):
+        """ORM-specific implementation to lookup the Data Access
+        Object (DAO) asynchronously.
+        """
+        return self.lookup_dao(checksum)
+
+    @abc.abstractmethod
+    def lookup_dao(self, checksum):
+        """ORM-specific implementation to lookup the Data Access
+        Object (DAO).
+        """
+        raise NotImplementedError("Subclasses must override this method.")
+
+    @abc.abstractmethod
+    def persist_dao(self, dao):
+        """ORM-specific implementation to lookup the Data Access
+        Object (DAO).
+        """
+        raise NotImplementedError("Subclasses must override this method.")
+
+    @abc.abstractmethod
     def exists(self, checksum):
         """Return a boolean indicating if a file with the given checksum
         exists.
         """
-        return self.model.objects.filter(checksum=checksum).exists()
+        raise NotImplementedError("Subclasses must override this method.")
 
     @ioc.inject('decrypter', 'BlobEncryptionBackend')
     def read(self, checksum, decrypter, decrypt=True):
         """Reads the full content of the :class:`Blob` identified by
         `checksum`.
         """
-        blob = self.model.objects.get(checksum=checksum)
+        blob = self.lookup_dao(checksum)
         with self.storage.open(checksum, 'rb') as f:
             buf = f.read()
             if blob.is_encrypted() and decrypt:
@@ -69,7 +98,7 @@ class AbstractBlobRepository:
         """Return a byte-sequence holding the slice specified by `length`
         and `offset`.
         """
-        blob = self.model.objects.get(checksum=checksum)
+        blob = await self.async_lookup_dao(checksum)
         if blob.is_encrypted():
             raise TypeError("Encrypted blobs can not be sliced.")
         return await self.storage.async_slice(checksum, length, offset)
@@ -91,3 +120,25 @@ class AbstractBlobRepository:
     def get_filesize(fp):
         """Return an unsigned integer indicating the size of file `fp`."""
         return os.path.getsize(fp)
+
+    def as_context(self, *args, **kwargs):
+        """Hook that is executed when a transaction is started."""
+        repo = type(self)()
+        repo.setup_context(*args, **kwargs)
+        return repo
+
+    def setup_context(self, *args, **kwargs):
+        """Hook to setup the context."""
+        pass
+
+    def teardown_context(self, *args, **kwargs):
+        """Hook to teardown the context."""
+        pass
+
+    def __enter__(self):
+        if self._in_transaction:
+            raise RuntimeError("Nested transactions are not supported.")
+        return self
+
+    def __exit__(self, cls, exc, tb):
+        pass
