@@ -43,9 +43,9 @@ class LocalDiskBackend(BaseStorageBackend):
                 async for chunk in reader:
                     await writer(chunk)
 
-    async def async_slice(self, path, length, offset=0, mode='r'):
+    async def async_slice(self, path, length, offset=0, binary=False):
         """Returns a slice of the blob represented by `path`."""
-        return self.slice(path, length, offset, mode)
+        return self.slice(path, length, offset, binary)
 
     def close(self, handler):
         """Flush and close the IO object.
@@ -94,9 +94,12 @@ class LocalDiskBackend(BaseStorageBackend):
             os.makedirs(os.path.dirname(dst))
         shutil.copy2(src, dst)
 
-    def slice(self, path, length, offset=0, mode='r'):
+    def slice(self, path, length, offset=0, binary=False):
         """Returns a slice of the blob represented by `path`."""
         self._validate_slice(length, offset)
+        mode = 'r'
+        if binary:
+            mode = 'rb'
         with open(self.storage_path(path), mode) as f:
             if offset:
                 f.seek(offset)
