@@ -6,6 +6,7 @@ from sqlalchemy import JSON
 from sqlalchemy import String
 from sqlalchemy.orm.exc import NoResultFound
 from unimatrix.lib import timezone
+from sqlalchemy.orm import relationship
 
 from ..abstract import BlobDAO
 from .base import Base
@@ -16,6 +17,7 @@ class Blob(Base, BlobDAO):
     __tablename__ = 'blobs'
     __module__ = 'unimatrix.ext.octet.orm'
     ObjectDoesNotExist = NoResultFound
+    references = relationship("BlobReference", back_populates='blob')
 
     #: A surrogate primary key.
     id = Column(
@@ -28,7 +30,7 @@ class Blob(Base, BlobDAO):
     #: SHA-256 hash of the blob content. If the :class:`Blob` is encrypted,
     #: the hash of the plain text.
     checksum = Column(
-        String(63),
+        String(64),
         nullable=False,
         unique=True,
         name='checksum'
